@@ -1,10 +1,8 @@
-from math import sqrt
-
 from numpy import (array, unravel_index, nditer, linalg, random, subtract, max,
-                   power, exp, pi, zeros, ones, arange, outer, meshgrid, dot,
-                   logical_and, mean, std, cov, argsort, linspace, transpose,
-                   einsum, prod, nan, sqrt, hstack, diff, argmin, multiply)
-from numpy import sum as npsum
+                   power, exp, zeros, ones, arange, outer, meshgrid, dot,
+                   logical_and, mean, cov, argsort, linspace, transpose,
+                   einsum, prod, nan, sqrt, hstack, diff, argmin, multiply,
+                   nanmean, nansum, tile, array_equal)
 from numpy.linalg import norm
 from collections import defaultdict, Counter
 from warnings import warn
@@ -25,16 +23,23 @@ import unittest
 
 
 def _build_iteration_indexes(data_len, num_iterations,
-                             verbose=False, random_generator=None):
+                             verbose=False, random_generator=None,
+                             use_epochs=False):
     """Returns an iterable with the indexes of the samples
     to pick at each iteration of the training.
 
-    If random_generator is not None, it must be an instalce
+    If random_generator is not None, it must be an instance
     of numpy.random.RandomState and it will be used
     to randomize the order of the samples."""
-    iterations = arange(num_iterations) % data_len
-    if random_generator:
-        random_generator.shuffle(iterations)
+    if use_epochs:
+        iterations_per_epoch = arange(data_len)
+        if random_generator:
+            random_generator.shuffle(iterations_per_epoch)
+        iterations = tile(iterations_per_epoch, num_iterations)
+    else:
+        iterations = arange(num_iterations) % data_len
+        if random_generator:
+            random_generator.shuffle(iterations)
     if verbose:
         return _wrap_index__in_verbose(iterations)
     else:
@@ -62,8 +67,6 @@ def _wrap_index__in_verbose(iterations):
 
 def fast_norm(x):
     """Returns norm-2 of a 1-D numpy array.
-
-    * faster than linalg.norm in case of 1-D arrays (numpy 1.9.2rc1).
     """
     return sqrt(dot(x, x.T))
 
@@ -119,7 +122,7 @@ class MiniSom(object):
             learning_rate(t) = learning_rate / (1 + t/T)
             where T is #num_iteration/2)
 
-        decay_function : function (default=None)
+        decay_function : function (default=asymptotic_decay)
             Function that reduces learning_rate and sigma at each iteration
             the default function is:
                         learning_rate / (1+t/(max_iterarations/2))
@@ -143,10 +146,15 @@ class MiniSom(object):
             Topology of the map.
             Possible values: 'rectangular', 'hexagonal'
 
-        activation_distance : string, optional (default='euclidean')
+        activation_distance : string, callable optional (default='euclidean')
             Distance used to activate the map.
             Possible values: 'euclidean', 'cosine', 'manhattan', 'chebyshev'
 
+            Example of callable that can be passed:
+
+            def euclidean(x, w):
+                return linalg.norm(subtract(x, w), axis=-1)
+
         random_seed : int, optional (default=None)
             Random seed to use.
         """
@@ -204,12 +212,15 @@ class MiniSom(object):
                               'manhattan': self._manhattan_distance,
                               'chebyshev': self._chebyshev_distance}
 
-        if activation_distance not in distance_functions:
-            msg = '%s not supported. Distances available: %s'
-            raise ValueError(msg % (activation_distance,
-                                    ', '.join(distance_functions.keys())))
+        if isinstance(activation_distance, str):
+            if activation_distance not in distance_functions:
+                msg = '%s not supported. Distances available: %s'
+                raise ValueError(msg % (activation_distance,
+                                        ', '.join(distance_functions.keys())))
 
-        self._activation_distance = distance_functions[activation_distance]
+            self._activation_distance = distance_functions[activation_distance]
+        elif callable(activation_distance):
+            self._activation_distance = activation_distance
 
     def get_weights(self):
         """Returns the weights of the neural network."""
@@ -316,9 +327,12 @@ class MiniSom(object):
         win : tuple
             Position of the winning neuron for x (array or tuple).
         t : int
-            Iteration index
+            rate of decay for sigma and learning rate
         max_iteration : int
-            Maximum number of training itarations.
+            If use_epochs is True:
+                Number of epochs the SOM will be trained for
+            If use_epochs is False:
+                Maximum number of iterations (one iteration per sample).
         """
         eta = self._decay_function(self._learning_rate, t, max_iteration)
         # sigma and learning rate decrease with the same rule
@@ -367,9 +381,11 @@ class MiniSom(object):
         pc_order = argsort(-pc_length)
         for i, c1 in enumerate(linspace(-1, 1, len(self._neigx))):
             for j, c2 in enumerate(linspace(-1, 1, len(self._neigy))):
-                self._weights[i, j] = c1*pc[pc_order[0]] + c2*pc[pc_order[1]]
+                self._weights[i, j] = c1*pc[:, pc_order[0]] + \
+                                      c2*pc[:, pc_order[1]]
 
-    def train(self, data, num_iteration, random_order=False, verbose=False):
+    def train(self, data, num_iteration,
+              random_order=False, verbose=False, use_epochs=False):
         """Trains the SOM.
 
         Parameters
@@ -378,14 +394,22 @@ class MiniSom(object):
             Data matrix.
 
         num_iteration : int
-            Maximum number of iterations (one iteration per sample).
+            If use_epochs is False, the weights will be
+            updated num_iteration times. Otherwise they will be updated
+            len(data)*num_iteration times.
+
         random_order : bool (default=False)
             If True, samples are picked in random order.
             Otherwise the samples are picked sequentially.
 
         verbose : bool (default=False)
-            If True the status of the training
-            will be printed at each iteration.
+            If True the status of the training will be
+            printed each time the weights are updated.
+
+        use_epochs : bool (default=False)
+            If True the SOM will be trained for num_iteration epochs.
+            In one epoch the weights are updated len(data) times and
+            the learning rate is constat throughout a single epoch.
         """
         self._check_iteration_number(num_iteration)
         self._check_input_len(data)
@@ -393,10 +417,18 @@ class MiniSom(object):
         if random_order:
             random_generator = self._random_generator
         iterations = _build_iteration_indexes(len(data), num_iteration,
-                                              verbose, random_generator)
+                                              verbose, random_generator,
+                                              use_epochs)
+        if use_epochs:
+            def get_decay_rate(iteration_index, data_len):
+                return int(iteration_index / data_len)
+        else:
+            def get_decay_rate(iteration_index, data_len):
+                return int(iteration_index)
         for t, iteration in enumerate(iterations):
+            decay_rate = get_decay_rate(t, len(data))
             self.update(data[iteration], self.winner(data[iteration]),
-                        t, num_iteration)
+                        decay_rate, num_iteration)
         if verbose:
             print('\n quantization error:', self.quantization_error(data))
 
@@ -413,7 +445,7 @@ class MiniSom(object):
 
         verbose : bool (default=False)
             If True the status of the training
-            will be printed at each iteration.
+            will be printed at each time the weights are updated.
         """
         self.train(data, num_iteration, random_order=True, verbose=verbose)
 
@@ -430,18 +462,32 @@ class MiniSom(object):
 
         verbose : bool (default=False)
             If True the status of the training
-            will be printed at each iteration.
+            will be printed at each time the weights are updated.
         """
         self.train(data, num_iteration, random_order=False, verbose=verbose)
 
-    def distance_map(self):
+    def distance_map(self, scaling='sum'):
         """Returns the distance map of the weights.
-        Each cell is the normalised sum of the distances between
-        a neuron and its neighbours. Note that this method uses
-        the euclidean distance."""
-        um = zeros((self._weights.shape[0],
-                    self._weights.shape[1],
-                    8))  # 2 spots more for hexagonal topology
+        If scaling is 'sum' (default), each cell is the normalised sum of
+        the distances between a neuron and its neighbours. Note that this
+        method uses the euclidean distance.
+
+        Parameters
+        ----------
+        scaling : string (default='sum')
+            If set to 'mean', each cell will be the normalized
+            by the average of the distances of the neighbours.
+            If set to 'sum', the normalization is done
+            by the sum of the distances.
+        """
+
+        if scaling not in ['sum', 'mean']:
+            raise ValueError(f'scaling should be either "sum" or "mean" ('
+                             f'"{scaling}" not valid)')
+
+        um = nan * zeros((self._weights.shape[0],
+                          self._weights.shape[1],
+                          8))  # 2 spots more for hexagonal topology
 
         ii = [[0, -1, -1, -1, 0, 1, 1, 1]]*2
         jj = [[-1, -1, 0, 1, 1, 1, 0, -1]]*2
@@ -460,7 +506,11 @@ class MiniSom(object):
                         w_1 = self._weights[x+i, y+j]
                         um[x, y, k] = fast_norm(w_2-w_1)
 
-        um = um.sum(axis=2)
+        if scaling == 'mean':
+            um = nanmean(um, axis=2)
+        if scaling == 'sum':
+            um = nansum(um, axis=2)
+
         return um/um.max()
 
     def activation_response(self, data):
@@ -503,14 +553,30 @@ class MiniSom(object):
         If the topographic error is 0, no error occurred.
         If 1, the topology was not preserved for any of the samples."""
         self._check_input_len(data)
-        if self.topology == 'hexagonal':
-            msg = 'Topographic error not implemented for hexagonal topology.'
-            raise NotImplementedError(msg)
         total_neurons = prod(self._activation_map.shape)
         if total_neurons == 1:
             warn('The topographic error is not defined for a 1-by-1 map.')
             return nan
+        if self.topology == 'hexagonal':
+            return self._topographic_error_hexagonal(data)
+        else:
+            return self._topographic_error_rectangular(data)
 
+    def _topographic_error_hexagonal(self, data):
+        """Return the topographic error for hexagonal grid"""
+        b2mu_inds = argsort(self._distance_from_weights(data), axis=1)[:, :2]
+        b2mu_coords = [[self._get_euclidean_coordinates_from_index(bmu[0]),
+                        self._get_euclidean_coordinates_from_index(bmu[1])]
+                       for bmu in b2mu_inds]
+        b2mu_coords = array(b2mu_coords)
+        b2mu_neighbors = [(bmu1 >= bmu2-1) & ((bmu1 <= bmu2+1))
+                          for bmu1, bmu2 in b2mu_coords]
+        b2mu_neighbors = [neighbors.prod() for neighbors in b2mu_neighbors]
+        te = 1 - mean(b2mu_neighbors)
+        return te
+
+    def _topographic_error_rectangular(self, data):
+        """Return the topographic error for rectangular grid"""
         t = 1.42
         # b2mu: best 2 matching units
         b2mu_inds = argsort(self._distance_from_weights(data), axis=1)[:, :2]
@@ -520,6 +586,15 @@ class MiniSom(object):
         distance = norm(dxdy, axis=1)
         return (distance > t).mean()
 
+    def _get_euclidean_coordinates_from_index(self, index):
+        """Returns the Euclidean coordinated of a neuron using its
+        index as the input"""
+        if index < 0:
+            return (-1, -1)
+        y = self._weights.shape[1]
+        coords = self.convert_map_to_euclidean((index % y, int(index/y)))
+        return coords
+
     def win_map(self, data, return_indices=False):
         """Returns a dictionary wm where wm[(i,j)] is a list with:
         - all the patterns that have been mapped to the position (i,j),
@@ -673,7 +748,7 @@ class TestMinisom(unittest.TestCase):
         distances = self.som._distance_from_weights(data)
         for i in range(len(data)):
             for j in range(len(weights)):
-                assert(distances[i][j] == norm(data[i] - weights[j]))
+                assert (distances[i][j] == norm(data[i] - weights[j]))
 
     def test_quantization_error(self):
         assert self.som.quantization_error([[5], [2]]) == 0.0
@@ -691,8 +766,18 @@ class TestMinisom(unittest.TestCase):
         assert self.som.topographic_error([[15]]) == 1.0
 
         self.som.topology = 'hexagonal'
-        with self.assertRaises(NotImplementedError):
-            assert self.som.topographic_error([[5]]) == 0.0
+        # 10 will have bmu_1 in (0, 4) and bmu_2 in (1, 3)
+        # which are in the same neighborhood on a hexagonal grid
+        self.som._weights[0, 4] = 10.0
+        self.som._weights[1, 3] = 9.0
+        # 3 will have bmu_1 in (2, 0) and bmu_2 in (1, 1)
+        # which are in the same neighborhood on a hexagonal grid
+        self.som._weights[2, 0] = 3.0
+        assert self.som.topographic_error([[10]]) == 0.0
+        assert self.som.topographic_error([[3]]) == 0.0
+        # True for both hexagonal and rectangular grids
+        assert self.som.topographic_error([[5]]) == 0.0
+        assert self.som.topographic_error([[15]]) == 1.0
         self.som.topology = 'rectangular'
 
     def test_quantization(self):
@@ -737,6 +822,38 @@ class TestMinisom(unittest.TestCase):
         som.train_random(data, 10, verbose=True)
         assert q1 > som.quantization_error(data)
 
+    def test_train_use_epochs(self):
+        som = MiniSom(5, 5, 2, sigma=1.0, learning_rate=0.5, random_seed=1)
+        data = array([[4, 2], [3, 1]])
+        q1 = som.quantization_error(data)
+        som.train(data, 10, use_epochs=True)
+        assert q1 > som.quantization_error(data)
+
+    def test_use_epochs_variables(self):
+        len_data = 100000
+        num_epochs = 100
+        random_gen = random.RandomState(1)
+        iterations = _build_iteration_indexes(len_data, num_epochs,
+                                              random_generator=random_gen,
+                                              use_epochs=True)
+        assert num_epochs*len_data == len(iterations)
+
+        # checks whether all epochs share the same order of indexes
+        first_epoch = iterations[0:len_data]
+        for i in range(num_epochs):
+            i_epoch = iterations[i*len_data:(i+1)*len_data]
+            assert array_equal(first_epoch, i_epoch)
+
+        # checks whether the decay_factor stays constant during one epoch
+        # and that its values range from 0 to num_epochs-1
+        decay_factors = []
+        for t, iteration in enumerate(iterations):
+            decay_factor = int(t / len_data)
+            decay_factors.append(decay_factor)
+        for i in range(num_epochs):
+            decay_factors_i_epoch = decay_factors[i*len_data:(i+1)*len_data]
+            assert decay_factors_i_epoch == [i]*len_data
+
     def test_random_weights_init(self):
         som = MiniSom(2, 2, 2, random_seed=1)
         som.random_weights_init(array([[1.0, .0]]))
@@ -746,8 +863,10 @@ class TestMinisom(unittest.TestCase):
     def test_pca_weights_init(self):
         som = MiniSom(2, 2, 2)
         som.pca_weights_init(array([[1.,  0.], [0., 1.], [1., 0.], [0., 1.]]))
-        expected = array([[[0., -1.41421356], [-1.41421356, 0.]],
-                          [[1.41421356, 0.], [0., 1.41421356]]])
+        expected = array([[[-1.41421356,  0.],
+                           [0.,  1.41421356]],
+                          [[0., -1.41421356],
+                           [1.41421356,  0.]]])
         assert_array_almost_equal(som._weights, expected)
 
     def test_distance_map(self):
@@ -759,9 +878,30 @@ class TestMinisom(unittest.TestCase):
         som._weights = array([[[1.,  0.], [0., 1.]], [[1., 0.], [0., 1.]]])
         assert_array_equal(som.distance_map(), array([[.5, 1.], [1., .5]]))
 
+        som = MiniSom(3, 3, 1, random_seed=1)
+        som._weights = array([[1, 0, 1], [0, 1, 0], [1, 0, 1]])
+        dist = array([[2/3, 3/5, 2/3], [3/5, 4/8, 3/5], [2/3, 3/5, 2/3]])
+        assert_array_equal(som.distance_map(scaling='mean'), dist/max(dist))
+
+        with self.assertRaises(ValueError):
+            som.distance_map(scaling='puppies')
+
     def test_pickling(self):
         with open('som.p', 'wb') as outfile:
             pickle.dump(self.som, outfile)
         with open('som.p', 'rb') as infile:
             pickle.load(infile)
         os.remove('som.p')
+
+    def test_callable_activation_distance(self):
+        def euclidean(x, w):
+            return linalg.norm(subtract(x, w), axis=-1)
+
+        data = random.rand(100, 2)
+        som1 = MiniSom(5, 5, 2, sigma=1.0, learning_rate=0.5,
+                       activation_distance=euclidean, random_seed=1)
+        som1.train_random(data, 10)
+        som2 = MiniSom(5, 5, 2, sigma=1.0, learning_rate=0.5, random_seed=1)
+        som2.train_random(data, 10)
+        # same state after training
+        assert_array_almost_equal(som1._weights, som2._weights)
